import {
  tryCallAsync
} from "./chunk-DN4AXDFO.js";

// src/farcaster/frames.tsx
import {
  FarcasterNetwork,
  FrameActionBody,
  Message,
  NobleEd25519Signer,
  getFarcasterTime,
  makeFrameAction
} from "@farcaster/core";
import { bytesToHex, hexToBytes } from "viem";
var signComposerAction = async function signComposerAction2(signerPrivateKey, actionContext) {
  const messageOrError = await tryCallAsync(
    () => createComposerActionMessageWithSignerKey(signerPrivateKey, actionContext)
  );
  if (messageOrError instanceof Error) {
    throw messageOrError;
  }
  const { message, trustedBytes } = messageOrError;
  return {
    untrustedData: {
      buttonIndex: 1,
      fid: actionContext.fid,
      messageHash: bytesToHex(message.hash),
      network: 1,
      state: Buffer.from(message.data.frameActionBody.state).toString(),
      timestamp: (/* @__PURE__ */ new Date()).getTime(),
      url: actionContext.url
    },
    trustedData: {
      messageBytes: trustedBytes
    }
  };
};
var signFrameAction = async (actionContext) => {
  const {
    frameButton,
    signer,
    buttonIndex,
    inputText,
    frameContext,
    state,
    url,
    target
  } = actionContext;
  if (!signer || signer.status === "pending_approval") {
    throw new Error("Missing signer fid to sign message");
  }
  const { message, trustedBytes } = await createFrameActionMessageWithSignerKey(
    signer.privateKey,
    {
      fid: signer.fid,
      buttonIndex,
      castId: {
        fid: frameContext.castId.fid,
        hash: hexToBytes(frameContext.castId.hash)
      },
      state: state !== void 0 ? Buffer.from(state) : void 0,
      url: Buffer.from(url),
      // it seems the message in hubs actually requires a value here.
      inputText: inputText !== void 0 ? Buffer.from(inputText) : void 0,
      address: actionContext.type === "tx-data" || actionContext.type === "tx-post" ? hexToBytes(actionContext.address) : void 0,
      transactionId: actionContext.type === "tx-post" ? hexToBytes(actionContext.transactionId) : void 0
    }
  );
  if (!message) {
    throw new Error("hub error");
  }
  const searchParams = new URLSearchParams({
    postType: actionContext.type === "tx-post" ? "post" : frameButton.action,
    postUrl: target ?? ""
  });
  return {
    searchParams,
    body: {
      untrustedData: {
        fid: signer.fid,
        url,
        messageHash: `0x${Buffer.from(message.hash).toString("hex")}`,
        timestamp: (/* @__PURE__ */ new Date()).getTime(),
        network: 1,
        buttonIndex: Number(message.data.frameActionBody.buttonIndex),
        castId: {
          fid: frameContext.castId.fid,
          hash: frameContext.castId.hash
        },
        inputText,
        state,
        address: actionContext.type === "tx-data" || actionContext.type === "tx-post" ? actionContext.address : void 0,
        transactionId: actionContext.type === "tx-post" ? actionContext.transactionId : void 0
      },
      trustedData: {
        messageBytes: trustedBytes
      }
    }
  };
};
async function createComposerActionMessageWithSignerKey(signerKey, { fid, state, url }) {
  const signer = new NobleEd25519Signer(Buffer.from(signerKey.slice(2), "hex"));
  const messageDataOptions = {
    fid,
    network: FarcasterNetwork.MAINNET
  };
  const message = await makeFrameAction(
    FrameActionBody.create({
      url: Buffer.from(url),
      buttonIndex: 1,
      state: Buffer.from(encodeURIComponent(JSON.stringify({ cast: state })))
    }),
    messageDataOptions,
    signer
  );
  if (message.isErr()) {
    throw message.error;
  }
  const messageData = message.value;
  const trustedBytes = Buffer.from(
    Message.encode(message._unsafeUnwrap()).finish()
  ).toString("hex");
  return { message: messageData, trustedBytes };
}
async function createFrameActionMessageWithSignerKey(signerKey, {
  fid,
  url,
  buttonIndex,
  castId,
  inputText,
  state,
  address,
  transactionId
}) {
  const signer = new NobleEd25519Signer(Buffer.from(signerKey.slice(2), "hex"));
  const messageDataOptions = {
    fid,
    network: FarcasterNetwork.MAINNET
  };
  const message = await makeFrameAction(
    FrameActionBody.create({
      url,
      buttonIndex,
      castId,
      state,
      inputText: inputText !== void 0 ? Buffer.from(inputText) : void 0,
      address,
      transactionId
    }),
    messageDataOptions,
    signer
  );
  if (message.isErr()) {
    return { message: null, trustedBytes: null };
  }
  const trustedBytes = Buffer.from(
    Message.encode(message._unsafeUnwrap()).finish()
  ).toString("hex");
  return { message: message.unwrapOr(null), trustedBytes };
}
function isFarcasterFrameContext(frameContext) {
  return "castId" in frameContext;
}
async function unsignedFrameAction(actionContext) {
  const {
    frameButton,
    target,
    frameContext,
    url,
    buttonIndex,
    state,
    inputText
  } = actionContext;
  const searchParams = new URLSearchParams({
    postType: actionContext.type === "tx-post" ? "post" : frameButton.action,
    postUrl: target ?? ""
  });
  return Promise.resolve({
    searchParams,
    body: {
      untrustedData: {
        url,
        ...isFarcasterFrameContext(frameContext) ? {
          castId: {
            fid: frameContext.castId.fid,
            hash: frameContext.castId.hash
          },
          network: 1,
          timestamp: getFarcasterTime()._unsafeUnwrap()
        } : {
          timestamp: Date.now()
        },
        buttonIndex,
        state,
        inputText,
        address: actionContext.type === "tx-data" || actionContext.type === "tx-post" ? actionContext.address : void 0,
        transactionId: actionContext.type === "tx-post" ? actionContext.transactionId : void 0
      },
      trustedData: {
        messageBytes: Buffer.from("").toString("hex")
      }
    }
  });
}

export {
  signComposerAction,
  signFrameAction,
  createComposerActionMessageWithSignerKey,
  createFrameActionMessageWithSignerKey,
  unsignedFrameAction
};
