import {
  useFreshRef
} from "./chunk-KTZM67PO.js";
import {
  ComposerActionUnexpectedResponseError,
  ComposerActionUserRejectedRequestError
} from "./chunk-LOKN5BYP.js";
import {
  isComposerFormActionResponse,
  mergeSearchParamsToUrl,
  tryCall,
  tryCallAsync
} from "./chunk-DN4AXDFO.js";
import {
  miniAppMessageSchema
} from "./chunk-AI2TJE4Y.js";
import "./chunk-6C3VEZWH.js";

// src/use-composer-action.ts
import { useCallback, useEffect, useMemo, useReducer, useRef } from "react";
function useComposerAction({
  actionState,
  enabled = true,
  proxyUrl,
  signer,
  url,
  fetch: fetchFunction,
  onError,
  onCreateCast,
  onSignature,
  onTransaction,
  resolveAddress,
  registerMessageListener = defaultRegisterMessageListener,
  onMessageRespond
}) {
  const onErrorRef = useFreshRef(onError);
  const [state, dispatch] = useReducer(composerActionReducer, {
    status: "idle",
    enabled
  });
  const registerMessageListenerRef = useFreshRef(registerMessageListener);
  const actionStateRef = useFreshRef(actionState);
  const onCreateCastRef = useFreshRef(onCreateCast);
  const onMessageRespondRef = useFreshRef(onMessageRespond);
  const onTransactionRef = useFreshRef(onTransaction);
  const onSignatureRef = useFreshRef(onSignature);
  const resolveAddressRef = useFreshRef(resolveAddress);
  const lastFetchActionArgRef = useRef(
    null
  );
  const signerRef = useFreshRef(signer);
  const fetchRef = useFreshRef(fetchFunction);
  const messageListener = useCallback(
    async (successState, message) => {
      if ("type" in message || message.method === "fc_createCast") {
        const cast = "type" in message ? message.data.cast : message.params.cast;
        const resultOrError = await tryCallAsync(
          () => onCreateCastRef.current({
            cast
          })
        );
        if (resultOrError instanceof Error) {
          onMessageRespondRef.current(
            {
              jsonrpc: "2.0",
              id: "method" in message ? message.id : null,
              error: {
                code: -32e3,
                message: resultOrError.message
              }
            },
            successState.response
          );
        }
        onMessageRespondRef.current(
          {
            jsonrpc: "2.0",
            id: "method" in message ? message.id : null,
            result: {
              success: true
            }
          },
          successState.response
        );
      } else if (message.method === "fc_requestWalletAction") {
        const addressOrError = await tryCallAsync(
          () => resolveAddressRef.current()
        );
        if (addressOrError instanceof Error) {
          tryCall(() => onErrorRef.current?.(addressOrError));
          onMessageRespondRef.current(
            {
              jsonrpc: "2.0",
              id: message.id,
              error: {
                code: -32e3,
                message: addressOrError.message
              }
            },
            successState.response
          );
          return;
        }
        if (!addressOrError) {
          return;
        }
        if (message.params.action.method === "eth_sendTransaction") {
          const action = message.params.action;
          const resultOrError = await tryCallAsync(
            () => onTransactionRef.current({
              action,
              address: addressOrError
            })
          );
          if (resultOrError instanceof Error) {
            tryCall(() => onErrorRef.current?.(resultOrError));
            onMessageRespondRef.current(
              {
                jsonrpc: "2.0",
                id: message.id,
                error: {
                  code: -32e3,
                  message: resultOrError.message
                }
              },
              successState.response
            );
            return;
          }
          if (!resultOrError) {
            const error = new ComposerActionUserRejectedRequestError();
            tryCall(() => onErrorRef.current?.(error));
            onMessageRespondRef.current(
              {
                jsonrpc: "2.0",
                id: message.id,
                error: {
                  code: -32e3,
                  message: error.message
                }
              },
              successState.response
            );
            return;
          }
          onMessageRespondRef.current(
            {
              jsonrpc: "2.0",
              id: message.id,
              result: {
                address: resultOrError.address,
                transactionHash: resultOrError.hash
              }
            },
            successState.response
          );
        } else if (message.params.action.method === "eth_signTypedData_v4") {
          const action = message.params.action;
          const resultOrError = await tryCallAsync(
            () => onSignatureRef.current({
              action,
              address: addressOrError
            })
          );
          if (resultOrError instanceof Error) {
            tryCall(() => onErrorRef.current?.(resultOrError));
            onMessageRespondRef.current(
              {
                jsonrpc: "2.0",
                id: message.id,
                error: {
                  code: -32e3,
                  message: resultOrError.message
                }
              },
              successState.response
            );
            return;
          }
          if (!resultOrError) {
            const error = new ComposerActionUserRejectedRequestError();
            tryCall(() => onErrorRef.current?.(error));
            onMessageRespondRef.current(
              {
                jsonrpc: "2.0",
                id: message.id,
                error: {
                  code: -32e3,
                  message: error.message
                }
              },
              successState.response
            );
            return;
          }
          onMessageRespondRef.current(
            {
              jsonrpc: "2.0",
              id: message.id,
              result: {
                address: resultOrError.address,
                signature: resultOrError.hash
              }
            },
            successState.response
          );
        } else {
          tryCall(
            () => onErrorRef.current?.(
              new Error(
                `Unknown fc_requestWalletAction action method: ${message.params.action.method}`
              )
            )
          );
        }
      } else {
        tryCall(() => onErrorRef.current?.(new Error("Unknown message")));
      }
    },
    [
      onCreateCastRef,
      onErrorRef,
      onMessageRespondRef,
      onSignatureRef,
      onTransactionRef,
      resolveAddressRef
    ]
  );
  const fetchAction = useCallback(
    async (arg) => {
      const currentSigner = arg.signer;
      if (currentSigner?.status !== "approved" && currentSigner?.status !== "impersonating") {
        await signerRef.current.onSignerlessFramePress();
        return;
      }
      dispatch({ type: "loading-url" });
      const signedDataOrError = await tryCallAsync(
        () => signerRef.current.signComposerAction(currentSigner.privateKey, {
          url: arg.url,
          state: arg.actionState,
          fid: currentSigner.fid
        })
      );
      if (signedDataOrError instanceof Error) {
        tryCall(() => onErrorRef.current?.(signedDataOrError));
        dispatch({ type: "error", error: signedDataOrError });
        return;
      }
      const proxiedUrl = mergeSearchParamsToUrl(
        arg.proxyUrl,
        new URLSearchParams({ postUrl: arg.url })
      );
      const actionResponseOrError = await tryCallAsync(
        () => (fetchRef.current || fetch)(proxiedUrl, {
          method: "POST",
          headers: {
            "Content-Type": "application/json"
          },
          body: JSON.stringify(signedDataOrError),
          cache: "no-cache"
        })
      );
      if (actionResponseOrError instanceof Error) {
        tryCall(() => onErrorRef.current?.(actionResponseOrError));
        dispatch({ type: "error", error: actionResponseOrError });
        return;
      }
      if (!actionResponseOrError.ok) {
        const error = new Error(
          `Unexpected response status ${actionResponseOrError.status}`
        );
        tryCall(() => onErrorRef.current?.(error));
        dispatch({ type: "error", error });
        return;
      }
      const actionResponseDataOrError = await tryCallAsync(
        () => actionResponseOrError.clone().json()
      );
      if (actionResponseDataOrError instanceof Error) {
        tryCall(() => onErrorRef.current?.(actionResponseDataOrError));
        dispatch({ type: "error", error: actionResponseDataOrError });
        return;
      }
      if (!isComposerFormActionResponse(actionResponseDataOrError)) {
        const error = new ComposerActionUnexpectedResponseError();
        tryCall(() => onErrorRef.current?.(error));
        dispatch({ type: "error", error });
        return;
      }
      dispatch({ type: "done", response: actionResponseDataOrError });
    },
    [fetchRef, onErrorRef, signerRef]
  );
  const stateRef = useFreshRef(state);
  const refetch = useCallback(() => {
    if (!stateRef.current.enabled || !lastFetchActionArgRef.current) {
      return Promise.resolve();
    }
    return fetchAction(lastFetchActionArgRef.current);
  }, [fetchAction, stateRef]);
  useEffect(() => {
    dispatch({ type: "enabled-change", enabled });
  }, [enabled]);
  useEffect(() => {
    if (!enabled) {
      return;
    }
    lastFetchActionArgRef.current = {
      actionState: actionStateRef.current,
      signer: signer.signer,
      url,
      proxyUrl
    };
    fetchAction(lastFetchActionArgRef.current).catch((e) => {
      onErrorRef.current?.(e instanceof Error ? e : new Error(String(e)));
    });
  }, [
    url,
    proxyUrl,
    signer.signer,
    enabled,
    fetchAction,
    actionStateRef,
    onErrorRef
  ]);
  useEffect(() => {
    if (state.status === "success") {
      return registerMessageListenerRef.current(
        state.response,
        messageListener.bind(null, state)
      );
    }
  }, [messageListener, registerMessageListenerRef, state]);
  return useMemo(() => {
    switch (state.status) {
      case "idle":
        return {
          status: "idle",
          data: void 0,
          error: void 0,
          refetch
        };
      case "loading":
        return {
          status: "loading",
          data: void 0,
          error: void 0,
          refetch
        };
      case "error":
        return {
          status: "error",
          data: void 0,
          error: state.error,
          refetch
        };
      default:
        return {
          status: "success",
          data: state.response,
          error: void 0,
          refetch
        };
    }
  }, [state, refetch]);
}
var defaultRegisterMessageListener = function defaultRegisterMessageListener2(formResponse, messageListener) {
  if (typeof window === "undefined") {
    console.warn(
      "@frames.js/render: You are using default registerMessageListener in an environment without window object"
    );
    return () => {
    };
  }
  const miniAppOrigin = new URL(formResponse.url).origin;
  const messageParserListener = (event) => {
    if (event.origin !== miniAppOrigin) {
      return;
    }
    const result = miniAppMessageSchema.safeParse(event.data);
    if (!result.success) {
      console.warn(
        "@frames.js/render: Invalid message received",
        event.data,
        result.error
      );
      return;
    }
    const message = result.data;
    messageListener(message).catch((e) => {
      console.error(`@frames.js/render:`, e);
    });
  };
  window.addEventListener("message", messageParserListener);
  return () => {
    window.removeEventListener("message", messageParserListener);
  };
};
function composerActionReducer(state, action) {
  if (action.type === "enabled-change") {
    if (action.enabled) {
      return {
        ...state,
        enabled: true
      };
    }
    return {
      status: "idle",
      enabled: false
    };
  }
  if (!state.enabled) {
    return state;
  }
  switch (action.type) {
    case "done":
      return {
        ...state,
        status: "success",
        response: action.response
      };
    case "loading-url":
      return {
        ...state,
        status: "loading"
      };
    case "error":
      return {
        ...state,
        status: "error",
        error: action.error
      };
    default:
      return state;
  }
}
export {
  defaultRegisterMessageListener,
  miniAppMessageSchema,
  useComposerAction
};
